from __future__ import annotations

from typing import Protocol


class PricingEngine(Protocol):
    def calculate(
        self,
        *,
        model: str,
        prompt_tokens: int,
        completion_tokens: int,
        price_book_version: str,
        billing_mode: str,
    ) -> float: ...
