"""Backward-compatible shim importing new provider capability helpers."""

from .providers.capabilities import ProviderCapabilitiesMap, list_capabilities

__all__ = ["ProviderCapabilitiesMap", "list_capabilities"]
