"""Gateway response utilities.

This module provides:
- errors: Stateless error formatting helpers (json_error, sse_error_event)
- sessions: Stateful Responses session management (ResponsesSession, save/get/drop)
"""

from ..response.errors import json_error, sse_error_event
from ..response.sessions import (
    ResponsesSession,
    SamplingConfig,
    continue_with_tool_outputs,
    drop_session,
    get_session,
    save_session,
    update_session_messages,
)

__all__ = [
    "json_error",
    "sse_error_event",
    "ResponsesSession",
    "SamplingConfig",
    "continue_with_tool_outputs",
    "drop_session",
    "get_session",
    "save_session",
    "update_session_messages",
]
