#!/usr/bin/env python
"""Simple test of the eb1 ensemble model."""

from ember.takumi.eb1 import run_eb1

# Test with a simple prompt
prompt = "What is 2 + 2?"
print(f"Prompt: {prompt}\n")

response = run_eb1(prompt)
print(f"EB1 Response: {response.data}\n")

# Check the raw output to see which model won
if hasattr(response, "raw_output") and response.raw_output:
    print("Debug info:")
    print(f"Winner model: {response.raw_output.get('winner_model')}")
    print(f"Judge model: {response.raw_output.get('judge_model')}")
    print(f"Judge reasoning: {response.raw_output.get('judge_response')}")
