from __future__ import annotations

from ..adapters.runtime import RuntimeOptions, build_default_services
from ..pipeline.container import Services


def bootstrap(
    *,
    adapters: dict[str, object] | None = None,
    options: RuntimeOptions | None = None,
) -> Services:
    # `adapters` parameter maintained for backward compatibility; ignored.
    return build_default_services(options=options)


__all__ = ["bootstrap"]
