from __future__ import annotations

from typing import Protocol


class ProviderRegistry(Protocol):
    def providers_for(self, canonical_model: str) -> list[str]: ...

    def capabilities(self) -> dict[str, dict[str, bool]]: ...

    def routing_preferences(self) -> dict[str, float]: ...
