from __future__ import annotations

from typing import Protocol


class Clock(Protocol):
    def now_s(self) -> int: ...


class IdFactory(Protocol):
    def new_request_id(self) -> str: ...

    def new_fbr_id(self) -> str: ...
