import ember

print("Attempting to instantiate eb1 model using ember.models.instance('eb1')")
try:
    eb1 = ember.models.instance("eb1")
    print("eb1 model instantiated successfully.")
    print("Calling eb1('who are you')")
    response = eb1("who are you")
    print("Response from eb1:")
    print(response)
except Exception as e:
    print(f"An error occurred: {e}")
