from __future__ import annotations

from typing import Protocol


class CredentialPool(Protocol):
    def acquire(self, org_id: str, provider: str) -> str | None: ...

    def release(self, org_id: str, provider: str, credential: str | None) -> None: ...
