from __future__ import annotations

from dataclasses import dataclass


@dataclass(frozen=True)
class OrgLimits:
    prompt_tokens: int
    completion_tokens: int
    prompt_multiplier: int


__all__ = ["OrgLimits"]
