from __future__ import annotations

from typing import Protocol

from keiro.gateway.infra.sur import SUR


class SurStore(Protocol):
    """Persists System Usage Records (SURs) for completed requests."""

    def persist(self, record: SUR) -> None: ...
