from __future__ import annotations

from typing import Protocol


class OrgPlanStore(Protocol):
    def get(self, org_id: str, *, fallback_tier: str) -> dict[str, object]: ...

    def billing_mode(self, plan: dict[str, object], provider: str) -> str: ...
