from __future__ import annotations

from typing import Protocol


class IdempotencyStore(Protocol):
    def claim(self, org_id: str, idem_key: str, request_id: str) -> bool: ...

    def get_request_id(self, org_id: str, idem_key: str) -> str | None: ...
