from ember.models.discovery.registry import register_provider as register_discovery_provider

from ._discovery import OpenAIDiscoveryAdapter
from ._responses import ResponsesHandler
from ._usage import usage_from_openai
from .provider import OpenAIProvider

__all__ = ["OpenAIProvider", "OpenAIDiscoveryAdapter", "ResponsesHandler", "usage_from_openai"]

register_discovery_provider(OpenAIDiscoveryAdapter())
